#include "stdafx.h"
#include "Pgm12a.h"
#include "PlaneFunction.h"
#include <cmath>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* PlaneFunction: Plane Wing Air Lift Pressure Function                    */
/*                                                                         */
/***************************************************************************/

PlaneFunction::PlaneFunction (int num, double &p1, double &p2) 
              : Function (num, p1, p2) {
 sprintf_s (caption, sizeof(caption),
  "Airplane Lift Pressure along Wing's Length; Width: %.0f' Length: %.0f'",
  p1, p2);

 pointsztens = 200;
 strcpy_s (title, sizeof(title),
  "The Lift Pressure Along the Length of the Plane Wing at 2 foot intervals");
 title_on_right = true;
}

/***************************************************************************/
/*                                                                         */
/* Create: construct the plot                                              */
/*                                                                         */
/***************************************************************************/
  
bool PlaneFunction::Create () {
 try {
  x = new double [num_pts];    // world space coords
  y = new double [num_pts];
  pts = new CPoint [num_pts];  // page space coords
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 double w = parm1 * .3048;   // convert width to meters
 double l = parm2 * .3048;   // convert length to meters
 double x0 = l - w;
 double p0 = 6.0E8;          // init air pressure newtons per meter squared
 double l2 = l * l;
 double l4 = l2 * l2;
 double l8 = l4 * l4;
 double p1 = p0 / (l8 * w * w);
 double yp = w / 2.;         // find pressure down the middle of the wing width
 double ef = yp / 4. - w / 6.;
 double yf = yp * (w - yp) / exp (ef * ef);
 double x1, x2, x3;
 xs = 0.;                    // begin at fuselage edge
 xe = l;                     // end at wing tip

 // if using hatch brush, reserve 2 points for polygon closure
 int numpts = num_pts;
 if (use_hatch_brush) numpts = num_pts - 2;
 dx = (xe - xs) / (numpts - 1);

 int i;
 // plot the graph in world space
 for (i=0; i<numpts; i++) {
  x[i] = xs + dx * i;
  x1 = x[i] - l;
  x2 = x[i] - 3.;
  x3 = x[i] - x0;
  y[i] = p1 * x[i] * x[i] * x1 * x1 * x2 * x2 * x3 * x3 * yf;
  // find the max and min values as points are created
  if (i) {
   if (y[i] > ymax) ymax = y[i];
   if (y[i] < ymin) ymin = y[i];
  }
  else {
   ymax = y[i];
   ymin = y[i];
  }
 }

 // if using hatch brush, force curve polygon closure
 if (use_hatch_brush) {
  i = numpts;
  x[i] = x[i-1];
  y[i] = 0;
  i++;
  x[i] = x[0];
  y[i] = 0;
 }

 double ysz, xsz;

 // scale image in world space coords to page space coords of draw_quad_size
 // so that the graph always is a reasonable physical size
 // by using the larger quadrant extent should the image lie in two quadrants
 if (ymax > 0)
  if (fabs (ymin) > ymax) ysz = fabs (ymin);
  else ysz = ymax;
 else ysz = fabs (ymin);
 yscale = draw_quad_size / ysz;
 
 if (xe > 0)
  if (fabs (xs) > xe) xsz = fabs (xs);
  else xsz = xe;
 else xsz = fabs (xs);
 xscale = draw_quad_size / xsz;

 // scale all plotted points to get from world space to page space
 for (i=0; i<num_pts; i++) {
  pts[i].x = (int) (x[i] * xscale);
  pts[i].y = (int) (y[i] * yscale);
 }
 // and scale the bounds as well
 xpmin = (int) (xs * xscale);
 xpmax = (int) (xe * xscale);
 ypmin = (int) (ymin * yscale);
 ypmax = (int) (ymax * yscale);

 // now set up the scaling to convert from page space coords into device space
 // the window extent includes the true drawing size plus borders on either end
 window_ext_y = (ypmax - ypmin) + 2 * border_size;
 window_ext_x = (xpmax - xpmin) + 2 * border_size;
 // the window origin is that point in page space that becomes the upper left
 // point in device space
 window_org_x = xpmin - border_size;
 window_org_y = ypmax + border_size;

 // now determine which axis are visible in device space
 xvisible = false;
 yvisible = false;
 if (window_ext_x + window_org_x >= 0 && window_org_x <= 0) yvisible = true;
 if (window_org_y >= 0 && window_org_y - window_ext_y <= 0) xvisible = true;

 delete [] x;
 delete [] y;
 x = NULL;
 y = NULL;

 // now we must display the tick marks and the text values represented by them
 // in case an axis is not actually visible, place the info on the
 // left or bottom line outlining the entire window

 // print the x axis ticks and scale
 // if not visible, use the bottom edge as the position to base the ticks
 xtics_ys = xvisible ? ticksize : window_org_y - window_ext_y - ticksize;
 xtics_ye = xvisible ? 0 : window_org_y - window_ext_y;

 // here use 2 feet as the units - get total possible number x ticks
 num_xtics_tot = (int) (parm2 / 2.);
 // allocate arrays for x axis coords and scale messages
 try {
  xtics_x = new int [num_xtics_tot];
  xtics_msg = new char* [num_xtics_tot];
  for (i=0; i<num_xtics_tot; i++) xtics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 int j = 0;
 for (i=1; i<=num_xtics_tot; i++) {
  int xx = (int) (i * 2 * .3048 * xscale); // cals x in page space
  if (xx >= window_org_x && xx <= window_org_x + window_ext_x) {// clip
   xtics_x[j] = xx;
   sprintf_s (xtics_msg[j], YSTRLEN, "%d", i * 2);
   j++;
  }
 }
 num_xtics = j;

 // now calculate the y axis ticks and scale values
 // if the y axis is not visible, use the left edge
 ytics_xs = yvisible ? ticksize : window_org_x + ticksize;
 ytics_xe = yvisible ? 0 : window_org_x;

 // come up with a reasonable set of evenly spaced ticks even though the
 // range of values could be anything - first use the the larger
 // portion, if the image lies in the two y quadrants
 ScaleYAxis ();

 // allocate arrays for y axis coord and scale message
 try {
  ytics_y = new int [num_ytics_tot];
  ytics_msg = new char* [num_ytics_tot];
  for (i=0; i<num_ytics_tot; i++) ytics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 // now calc the y axis - handle clipping
 j = 0;
 // calc the ticks on the y axis - handle clipping an don't print at y = 0
 for (i=-num_ytics_tot/2; i<=num_ytics_tot/2; i++) {
  // calculate the y position in page space coords
  int yy = (int) (ybase * i * yscale);
  if (yy <= window_org_y && yy >= window_org_y - window_ext_y && i) {
   ytics_y[j] = yy;
   if (!istenths) sprintf_s (ytics_msg[j], YSTRLEN, "%d", i);
   else sprintf_s (ytics_msg[j], YSTRLEN, "%.1f", i * .2);
   j++;
  }
 }
 num_ytics = j;
 
 // now display the y units scale in the upper right corner , second line
 sprintf_s (yscale_msg, sizeof(yscale_msg), "(Y units: %.2E n/m/m)", ybase);
 yscale_msg_up = true;

 return true;
}
