#include "stdafx.h"
#include "Pgm12a.h"
#include "SpringFunction.h"
#include <cmath>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* SpringFunction: Car shock absorber effectiveness                        */
/*                                                                         */
/***************************************************************************/

SpringFunction::SpringFunction (int num, double &p1, double &p2) 
               : Function (num, p1, p2) {
 sprintf_s (caption, sizeof(caption),
          "Shock Absorber Action vs Time: Car Weight: %.0f; Pothole: %.0f\"",
          p1, p2);
 pointsztens = 160;
 strcpy_s (title, sizeof(title),
  "Shock Absorber Effectiveness - Height Displacement vs Seconds after the Bump");
 title_on_right = true;
 use_hatch_brush = false;
}

/***************************************************************************/
/*                                                                         */
/* Create: construct the plot                                              */
/*                                                                         */
/***************************************************************************/
  
bool      SpringFunction::Create () {
 try {
  x = new double [num_pts];      // the plotted world coords
  y = new double [num_pts];
  pts = new CPoint [num_pts];    // the page space coords
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 double m = parm1 * 373.24172;    // convert to grams
 double x0 = parm2 / 12. * .3048; // convert inches to meters
 double c = 1.4E7;                // spring damping coefficient grams/second
 double k = 1.25E9;               // spring constant grams/second/second
 double n = c / (2. * m);
 double t1 = k / m;
 double t2 = c * c / (4. * m * m);
 if (t1 < t2) {                   // validity check
  MessageBeep (0);
  AfxMessageBox ("Error: cannot use this car weight - try another", 
                  MB_OK | MB_ICONSTOP);
  return false;
 }
 double p = sqrt (t1 - t2);       // valid so calc sqrt
 
 xs = 0;                          // time 0
 xe = .5;                         // half second later

 dx = (xe - xs) / (num_pts - 1);
 int i;

 // plot the graph in world space
 for (i=0; i<num_pts; i++) {
  x[i] = xs + dx * i;
  y[i] = exp (-n * x[i]) * (x0 * cos (p * x[i]) + x0 * n / p * sin (p * x[i]));
  // find the max and min values as points are created
  if (i) {
   if (y[i] > ymax) ymax = y[i];
   if (y[i] < ymin) ymin = y[i];
  }
  else {
   ymax = y[i];
   ymin = y[i];
  }
 }

 double ysz, xsz;

 // scale image in world space coords to page space coords of draw_quad_size
 // so that the graph always is a reasonable physical size
 // by using the larger quadrant extent should the image lie in two quadrants
 if (ymax > 0)
  if (fabs (ymin) > ymax) ysz = fabs (ymin);
  else ysz = ymax;
 else ysz = fabs (ymin);
 yscale = draw_quad_size / ysz;
 
 if (xe > 0)
  if (fabs (xs) > xe) xsz = fabs (xs);
  else xsz = xe;
 else xsz = fabs (xs);
 xscale = draw_quad_size / xsz;

 // scale all plotted points to get from world space to page space
 for (i=0; i<num_pts; i++) {
  pts[i].x = (int) (x[i] * xscale);
  pts[i].y = (int) (y[i] * yscale);
 }
 // and scale the bounds as well
 xpmin = (int) (xs * xscale);
 xpmax = (int) (xe * xscale);
 ypmin = (int) (ymin * yscale);
 ypmax = (int) (ymax * yscale);

 // now set up the scaling to convert from page space coords into device space
 // the window extent includes the true drawing size plus borders on either end
 window_ext_y = (ypmax - ypmin) + 2 * border_size;
 window_ext_x = (xpmax - xpmin) + 2 * border_size;
 // the window origin is that point in page space that becomes the upper left
 // point in device space
 window_org_x = xpmin - border_size;
 window_org_y = ypmax + border_size;

 // now determine which axis are visible in device space
 xvisible = false;
 yvisible = false;
 if (window_ext_x + window_org_x >= 0 && window_org_x <= 0) yvisible = true;
 if (window_org_y >= 0 && window_org_y - window_ext_y <= 0) xvisible = true;

 delete [] x;
 delete [] y;
 x = NULL;
 y = NULL;

 // now calculate the x axis tick marks and the text values represented by them
 // in case an axis is not actually visible, place the info on the
 // left or bottom line outlining the entire window

 // if not visible, use the bottom edge as the position to base the ticks
 xtics_ys = xvisible ? ticksize : window_org_y - window_ext_y - ticksize;
 xtics_ye = xvisible ? 0 : window_org_y - window_ext_y;
 
 // here .1 second units - allocate arrays for x coord and x scale msgs
 num_xtics_tot = (int) (xe * 10);
 try {
  xtics_x = new int [num_xtics_tot];
  xtics_msg = new char* [num_xtics_tot];
  for (i=0; i<num_xtics_tot; i++) xtics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }
 
 int j = 0;
 double ix;
 for (ix=.1; ix<=xe; ix+=.1) {
  // calc position of x axis ticks in world coords and convert to page space
  int xx = (int) (ix * xscale);
  // clip any outside page space
  if (xx >= window_org_x && xx <= window_org_x + window_ext_x) {
   xtics_x[j] = xx;
   sprintf_s (xtics_msg[j], YSTRLEN, "%.1f", ix);
   j++;
  }
 }
 num_xtics = j;

 // now calculate the y axis ticks and scale values
 // if the y axis is not visible, use the left edge
 ytics_xs = yvisible ? ticksize : window_org_x + ticksize;
 ytics_xe = yvisible ? 0 : window_org_x;

 // come up with a reasonable set of evenly spaced ticks even though the
 // range of values could be anything - first use the the larger
 // portion, if the image lies in the two y quadrants
 ScaleYAxis ();

 // allocate arrays for y axis coord and scale message
 try {
  ytics_y = new int [num_ytics_tot];
  ytics_msg = new char* [num_ytics_tot];
  for (i=0; i<num_ytics_tot; i++) ytics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 // now calc the y axis - handle clipping
 j = 0;
 for (i=-num_ytics_tot/2; i<=num_ytics_tot/2; i++) {
  // calculate the y position in world coords and convert to page space coords
  int yy = (int) (ybase * i * yscale);
  // clip if outside page space and avoid printing at y = 0
  if (yy <= window_org_y && yy >= window_org_y - window_ext_y && i) {
   ytics_y[j] = yy;
   if (!istenths) sprintf_s (ytics_msg[j], YSTRLEN, "%d", i);
   else sprintf_s (ytics_msg[j], YSTRLEN, "%.1f", i * .2);
   j++;
  }
 }
 num_ytics = j;

 // now install the y units scale message in the lower right corner
 yscale_msg_up = false;
 sprintf_s (yscale_msg, sizeof(yscale_msg),
            "(Y units: %.1f inches)", ybase * 12 / .3048);

 return true;
}
