#include "stdafx.h"
#include "CDib.h"
#include <math.h>

/***************************************************************************/
/*                                                                         */
/* CDib: Load the DIB into a memory CDib object from the CFile             */
/*                                                                         */
/***************************************************************************/

          CDib::CDib (CFile *ptrfile) {
 ptrdib = NULL;         // set for none loaded yet
 hdib = 0;
 ReadDibFile (ptrfile); // attempt the loading of the DIB file
}

/***************************************************************************/
/*                                                                         */
/* ~CDib: Delete the CDib object                                           */
/*                                                                         */
/***************************************************************************/

          CDib::~CDib () {
 if (ptrdib) {
  GlobalUnlock (hdib);  // remove allocated objects
  GlobalFree (hdib);
 }
}

/***************************************************************************/
/*                                                                         */
/* IsValid: returns TRUE is DIB is loaded properly                         */
/*                                                                         */
/***************************************************************************/

BOOL      CDib::IsValid () {
 return (ptrdib) ? TRUE : FALSE;
}

/***************************************************************************/
/*                                                                         */
/* ReadDibFile: loads a .BMP file into a dib                               */
/*                                                                         */
/***************************************************************************/

void      CDib::ReadDibFile (CFile *ptrfile) {
 BITMAPFILEHEADER bmfh;
 UINT             actsz;

 ptrdib = NULL;
 hdib = 0;
 // input the BITMAPFILEHEADER structure
 actsz = ptrfile->Read ((LPSTR) &bmfh, sizeof (BITMAPFILEHEADER));
 if (actsz != sizeof (BITMAPFILEHEADER)) return;
 // verify it is a .BMP file
 if (bmfh.bfType != * (WORD *) "BM") return;
 
 // calculate the size of the file less the file header
 // and allocate memory for the dib
 dwDibSize = bmfh.bfSize - sizeof (BITMAPFILEHEADER);
 hdib = GlobalAlloc (GMEM_MOVEABLE, dwDibSize);
 ptrdib = (BYTE*) GlobalLock (hdib);
 if (!ptrdib)  return;
 
 // read in the rest of the DIB - 
 // could be huge amount of bytes for a large .BMP file
 actsz = ptrfile->Read ((LPSTR) ptrdib, dwDibSize);
 if (actsz != dwDibSize) { // error, wrong number of bytes
   GlobalUnlock (hdib);
   GlobalFree (hdib);
   ptrdib = NULL;
   return;
  }
 // check the validity of the size of the info header - fail OS2 bitmaps
 if (((BITMAPINFOHEADER*) ptrdib)->biSize <= sizeof(BITMAPCOREHEADER)) {
  GlobalUnlock (hdib);
  GlobalFree (hdib);
  ptrdib = NULL;
  return;
 }
 // now determine the number of colors in the color palette
 WORD bitcount = ((BITMAPINFOHEADER*) ptrdib)->biBitCount; // get the bitcount
  
 // there may be a possibility that the biClrUsed field is really there
 // assume biClrUsed is not present and fill it if so
 numcolors = ((BITMAPINFOHEADER*) ptrdib)->biClrUsed;
 // since biClrUsed itself may be 0, as it often is, determine from bit count
 if (numcolors == 0 && bitcount != 24) numcolors = 1L << bitcount;
 // we are not handling the 24-color modes
 if (numcolors == 0) { // no color table is present, so I consider this DIB 
                       // to be invalid - you could modify the palette 
                       // construction to permit the DIB to be shown using
                       // the default palette, if desired
  GlobalUnlock (hdib);
  GlobalFree (hdib);
  ptrdib = NULL;
  return;
 }
}

/***************************************************************************/
/*                                                                         */
/* MakePalette: constructs logical color palette                           */
/*                                                                         */
/***************************************************************************/

void      CDib::MakePalette (CPalette *ptrpalette) {
 LPLOGPALETTE ptrpal;
 HANDLE       hpal;                    // handle for the palette in memory

 if (!IsValid ()) return;
 
 // allocate a LOGPALETTE for the # of colors in use such as 16-color,256-color
 hpal = GlobalAlloc (GHND, sizeof (LOGPALETTE) + 
                     numcolors * sizeof (PALETTEENTRY));
 ptrpal = (LPLOGPALETTE) GlobalLock (hpal);
 if (!ptrpal) return;

 // fill up the LOGPALETTE structure by copying from the dib color array
 ptrpal->palNumEntries = (unsigned short) numcolors;
 ptrpal->palVersion = 0x300;
  
 LPBITMAPINFO ptrinfo = (LPBITMAPINFO) ptrdib;
 for (int i=0; i<(int)numcolors; i++) {
  ptrpal->palPalEntry[i].peRed = ptrinfo->bmiColors[i].rgbRed;
  ptrpal->palPalEntry[i].peGreen = ptrinfo->bmiColors[i].rgbGreen;
  ptrpal->palPalEntry[i].peBlue = ptrinfo->bmiColors[i].rgbBlue;
  ptrpal->palPalEntry[i].peFlags = 0;
 }

 // fill in the caller's palette by creating the palette from LOGPALETTE
 ptrpalette->CreatePalette (ptrpal);
 
 // now delete the LOGPALETTE which is no longer needed
 GlobalUnlock (hpal);
 GlobalFree (hpal);
}

/***************************************************************************/
/*                                                                         */
/* MakeIdentityPalette: constructs a logical identity color palette        */
/*                      must be a 256-color DIB                            */
/*                                                                         */
/***************************************************************************/

BOOL      CDib::MakeIdentityPalette (CPalette *ptrpalette) {
 if (!IsValid ()) return FALSE;
 // verify it is a 256-color DIB
 LPBITMAPINFOHEADER ptrbm = (LPBITMAPINFOHEADER) GetBitmapInfo ();
 if (ptrbm->biBitCount !=8) return FALSE;

 LOGPALETTE  *ptrpal;
 CPalette    *ptroldpal;
 CDC          dc;

 // get access to a screen DC - verify it is a palette device
 if (!dc.CreateDC ("DISPLAY", NULL, NULL, NULL)) return FALSE;
 else if ((~dc.GetDeviceCaps (RASTERCAPS) & RC_PALETTE)) {
  dc.DeleteDC ();
  return FALSE;
 } 
 
 // check that screen is in 256 color mode
 int num_syscolors = dc.GetDeviceCaps (NUMCOLORS);
 if (num_syscolors > 256) {
  dc.DeleteDC ();
  return FALSE;
 }
 
 // retrieve the number of palette entries
 int num_palette_entries = dc.GetDeviceCaps (SIZEPALETTE);

 // force Palette Manager to reset the master color table
 // so the next palette that is realized gets its colors
 // mapped into the order of use in the logical palette
 SetSystemPaletteUse (dc.GetSafeHdc (), SYSPAL_NOSTATIC);
 SetSystemPaletteUse (dc.GetSafeHdc (), SYSPAL_STATIC);

 // allocate a LOGPALETTE for the # of colors in use should be here 256-color
 ptrpal = (LOGPALETTE*) malloc (sizeof (LOGPALETTE) + 
                                numcolors * sizeof (PALETTEENTRY));
 if (!ptrpal) {
  dc.DeleteDC ();
  return FALSE;
 }

 // construct the basic DIB logical palette
 // fill up the LOGPALETTE structure by copying from the dib color array
 ptrpal->palNumEntries = (unsigned short) numcolors;
 ptrpal->palVersion = 0x300;

 int i;  
 LPBITMAPINFO ptrinfo = (LPBITMAPINFO) ptrdib;
 // copy the DIB's RBG colors into the logical palette
 for (i=0; i<(int)numcolors; i++) {
  ptrpal->palPalEntry[i].peRed   = ptrinfo->bmiColors[i].rgbRed;
  ptrpal->palPalEntry[i].peGreen = ptrinfo->bmiColors[i].rgbGreen;
  ptrpal->palPalEntry[i].peBlue  = ptrinfo->bmiColors[i].rgbBlue;
  ptrpal->palPalEntry[i].peFlags = 0;
 }

 // fill in the caller's palette by creating the palette from LOGPALETTE
 ptrpalette->CreatePalette (ptrpal);
 if (!ptrpalette) {
  dc.DeleteDC ();
  free (ptrpal);
  return FALSE;
 }

 // install our new palette, realize it to map the logical palette
 // into the master palette, then remove the palette from the DC
 ptroldpal = dc.SelectPalette (ptrpalette, FALSE);
 dc.RealizePalette ();
 dc.SelectPalette (ptroldpal, FALSE);

 // now construct a copy of the altered, mapped real palette
 // by getting all 256 entries
 PALETTEENTRY pe[256];
 GetSystemPaletteEntries (dc.GetSafeHdc (), 0, num_palette_entries, pe);

 // alter the 20 system colors' flags to normal
 // (the first 10 and last 10 entries)
 // and set our needed colors's flags (the remaining 236) to no collapse
 // so that those slots will remain asis and not get remapped to a 
 // possible duplicate, earlier palette entry
 for (i=0; i<num_syscolors/2; i++) pe[i].peFlags = 0;
 for (; i< num_palette_entries - num_syscolors/2; i++) pe[i].peFlags = PC_NOCOLLAPSE;
 for (; i< num_palette_entries; i++) pe[i].peFlags = 0;

 // handle any possible resizing of the palette
 ptrpalette->ResizePalette (num_palette_entries);
 // copy the identity palette in pe into our palette
 ptrpalette->SetPaletteEntries (0, num_palette_entries, pe);

 // next, alter the DIB's color table to match the logical identity palette
 // get a pointer to the DIB's color table
 RGBQUAD *ptrcolortab = (LPRGBQUAD)(ptrdib + sizeof(BITMAPINFOHEADER));
 RGBQUAD *ptrsave = ptrcolortab;
 // for each of the colors in the original DIB color table, find its
 // matching palette index in the identity palette
 BYTE map[256];
 for (i=0; i<256; i++, ptrcolortab++)
  map[i] = (BYTE) ptrpalette->GetNearestPaletteIndex (
   RGB(ptrcolortab->rgbRed, ptrcolortab->rgbGreen, ptrcolortab->rgbBlue));
 
 ptrcolortab = ptrsave;

 // replace each color byte (for each pixel) in the DIB with the corresponding
 // mapped color index
 // here is why only 256-color DIB can be converted, if in 16-color, each byte
 // represents more tham one pixel
 BYTE *ptrb = GetDibBitsAddr();
 i = GetDibHeight () * GetDibWidth ();
 while (i--) {
  *ptrb = map[*ptrb];
  ptrb++;
 }
 // and rewrite the DIB's color table so that it agrees with the identity palette
 ptrpalette->GetPaletteEntries (0, 256, pe);
 for (i=0; i<256; i++) {
  ptrcolortab->rgbRed   = pe[i].peRed;
  ptrcolortab->rgbGreen = pe[i].peGreen;
  ptrcolortab->rgbBlue  = pe[i].peBlue;
  ptrcolortab++;
 }

 dc.DeleteDC ();
 free (ptrpal);
 return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* GetDibWidth: returns pixel width of image                               */
/*                                                                         */
/***************************************************************************/

int       CDib::GetDibWidth () {
 return (!IsValid ()) ? 0 : ((BITMAPINFOHEADER*) ptrdib)->biWidth;
}

/***************************************************************************/
/*                                                                         */
/* GetDibHeight: returns pixel height of image                             */
/*                                                                         */
/***************************************************************************/

int       CDib::GetDibHeight () {
 return (!IsValid ()) ? 0 : ((BITMAPINFOHEADER*) ptrdib)->biHeight;
}

/***************************************************************************/
/*                                                                         */
/* GetDibBitsAddr: returns the addr of the color bits of the image         */
/*                                                                         */
/***************************************************************************/

BYTE*  CDib::GetDibBitsAddr () {
 DWORD colortablesize;
 
 if (!IsValid ()) return NULL;

 // calculate the offset from the info header over the color table
 colortablesize = numcolors * sizeof (RGBQUAD);
 // and return the address of the bits portion
 return ptrdib + ((BITMAPINFOHEADER*) ptrdib)->biSize + colortablesize;
}

/***************************************************************************/
/*                                                                         */
/* GetBitmapInfo:  returns the addr of the BITMAPINFO data                 */
/*                                                                         */
/***************************************************************************/

LPBITMAPINFO CDib::GetBitmapInfo () {
 return (!IsValid ()) ? NULL : (LPBITMAPINFO) ptrdib;
}

/***************************************************************************/
/*                                                                         */
/* CDib: construct gray DIB from existing CDib                             */
/*                                                                         */
/***************************************************************************/

          CDib::CDib (CDib *ptrcopy) {
 ptrdib = NULL;         // set for none loaded yet
 hdib = 0;

 if (!ptrcopy || !ptrcopy->IsValid ()) return;

 dwDibSize = ptrcopy->dwDibSize;   // set size and number of colors
 numcolors = ptrcopy->numcolors;

 hdib = GlobalAlloc (GMEM_MOVEABLE, dwDibSize); // allocate DIB copy 
 if (!hdib) return;
 
 ptrdib = (BYTE*) GlobalLock (hdib);            // lock ptr to new DIB
 memcpy (ptrdib, ptrcopy->ptrdib, dwDibSize);   // copy DIB
 if (numcolors == 256) GrayDib (ptrcopy);       // and convert to gray
 // do not convert any other types to gray scale
}

/***************************************************************************/
/*                                                                         */
/* GrayDib: convert this DIB into a gray copy of original CDib             */
/*                                                                         */
/***************************************************************************/

void      CDib::GrayDib (CDib *ptrcopy) {
 CDC dc;
 int i;
 // get access to color tables of original and this one to be grayed
 LPBITMAPINFO ptrinfocopy = ptrcopy->GetBitmapInfo ();
 LPBITMAPINFO ptrinfo     = GetBitmapInfo ();

 // get access to a screen DC - verify it is in 256 mode
 // then copy and install the 20 system colors
 if (dc.CreateDC ("DISPLAY", NULL, NULL, NULL)) {
  int num_syscolors = dc.GetDeviceCaps (NUMCOLORS);
  if (num_syscolors == 256) {
   PALETTEENTRY pe[256];
   GetSystemPaletteEntries (dc.GetSafeHdc (), 0, 256, pe);
   for (i=0; i<10; i++) {
    ptrinfo->bmiColors[i].rgbBlue  = pe[i].peBlue;
    ptrinfo->bmiColors[i].rgbRed   = pe[i].peRed;
    ptrinfo->bmiColors[i].rgbGreen = pe[i].peGreen;
    ptrinfo->bmiColors[i+246].rgbBlue  = pe[i+246].peBlue;
    ptrinfo->bmiColors[i+246].rgbRed   = pe[i+246].peRed;
    ptrinfo->bmiColors[i+246].rgbGreen = pe[i+246].peGreen;
   }
  }
  dc.DeleteDC ();
 }
 // assign a uniform gray palette to all remaining entries
 for (i=10; i<246; i++) {
  ptrinfo->bmiColors[i].rgbBlue = (BYTE) i;
  ptrinfo->bmiColors[i].rgbRed = (BYTE) i;
  ptrinfo->bmiColors[i].rgbGreen = (BYTE) i;
 }
 // convert all bits
 BYTE *ptrbits = GetDibBitsAddr ();
 // set i to the total number of bits
 i = GetDibHeight () * GetDibWidth ();
 BYTE color;
 int j;
 int ncolor = 0;            // number of unique colors used in DIB
 int histogram[256] = {0};  // frequency of each color palette IDs in use
 while (i--) {
  color = *ptrbits;
  // gray color by straight average of RGB intensities
  j = (((int) ptrinfocopy->bmiColors[color].rgbRed) +
       ((int) ptrinfocopy->bmiColors[color].rgbBlue) +
       ((int) ptrinfocopy->bmiColors[color].rgbGreen)) / 3;
  // force into gray scale range
  if (j < 10) j = 10;
  else if (j>244) j = 244;
  // add to histogram
  if (histogram[j]==0) ncolor++;
  histogram[j]++;
  *ptrbits++ = (BYTE) j;
 }
 // here histogram and ncolor contain a snapshot of DIB color usage
}