#include "stdafx.h"
#pragma hdrstop
#include <math.h>
#include "position.h"            // class definition

/***************************************************************************/
/*                                                                         */
/* Position Class Functions: Store and Manipulate a Position - Lat, Lon    */
/*                                                                         */
/***************************************************************************/

/***************************************************************************/
/*                                                                         */
/* Position (); Null Constructor - assigns 0,0                             */
/*                                                                         */
/***************************************************************************/

Position::Position () {
 Latitude  = 0;
 Longitude = 0;
}

/***************************************************************************/
/*                                                                         */
/* Position (long, long) - constructs Lat, Lon from long values            */
/*                                                                         */
/***************************************************************************/

Position::Position (long lat, long lon) {
 // verify in range
 if (lat > TOPLAT) lat = TOPLAT;
 else if (lat < BOTLAT) lat = BOTLAT;

 while (lon > MAXLON) lon -= MAXLON;
 while (lon < 0 ) lon += MAXLON;

 // assign valid position
 Latitude = lat;
 Longitude = lon;
}

/***************************************************************************/
/*                                                                         */
/* Position - constructs Lat, Lon from deg, min, sec, and string Direction */
/*                                                                         */
/***************************************************************************/

Position::Position (long& latdeg, long& latmin, long& latsec, char *latdir,
                        long& londeg, long& lonmin, long& lonsec, char *londir){
 SetLat (latdeg, latmin, latsec, latdir);
 SetLon (londeg, lonmin, lonsec, londir);
}

/***************************************************************************/
/*                                                                         */
/* Position - constructs Lat, Lon from deg, min, sec, where negatives = S/W*/
/*                                                                         */
/***************************************************************************/

Position::Position (long& latdeg, long& latmin, long& latsec,
                       long& londeg, long& lonmin, long& lonsec) {
 SetLat (latdeg, latmin, latsec);
 SetLon (londeg, lonmin, lonsec);
}

/***************************************************************************/
/*                                                                         */
/* Position (Position&) copy constructor                                   */
/*                                                                         */
/***************************************************************************/

Position::Position (Position& pos) { // copy constructor
 Latitude  = pos.Latitude;
 Longitude = pos.Longitude;
}

/***************************************************************************/
/*                                                                         */
/* SetLat (long) - sets Lat from a long latitude value                     */
/*                                                                         */
/***************************************************************************/

void Position::SetLat   (long lat) {
 if (lat > TOPLAT) lat = TOPLAT;      // force into in range
 else if (lat < BOTLAT) lat = BOTLAT;
 Latitude = lat;                      // and store latitude
}

/***************************************************************************/
/*                                                                         */
/* SetLat - sets Lat from deg, min, sec and string direction N or S        */
/*                                                                         */
/***************************************************************************/

void Position::SetLat   (long& deg, long& min, long& sec, char *dir) {
 long lat = (deg<90)? abs(deg)*3600L + abs(min)*60L + abs(sec): 90*3600L;
 if (dir[0] == 'S' || dir[0] == 's') lat = -lat;
 if (lat > TOPLAT) lat = TOPLAT;      // force into in range
 else if (lat < BOTLAT) lat = BOTLAT;
 Latitude = lat;                      // and store latitude
}

/***************************************************************************/
/*                                                                         */
/* SetLat - sets Lat from deg, min, sec where neg values = S lat           */
/*                                                                         */
/***************************************************************************/

void Position::SetLat   (long& deg, long& min, long& sec) {
 // N latitudes are positive, S latitudes are negative
 long lat = abs(deg)*3600L + abs(min)*60L + abs(sec);
 if (deg<0) lat = -lat;
 if (lat > TOPLAT) lat = TOPLAT;      // force into in range
 else if (lat < BOTLAT) lat = BOTLAT;
 Latitude = lat;                      // and store latitude
}

/***************************************************************************/
/*                                                                         */
/* SetLon (long) - sets Lon from a long value                              */
/*                                                                         */
/***************************************************************************/

void Position::SetLon   (long lon) {
 while (lon > MAXLON) lon -= MAXLON;
 while (lon < 0 ) lon += MAXLON;
 Longitude = lon;
}

/***************************************************************************/
/*                                                                         */
/* SetLon - sets Lon from deg, min, sec, and string direction W or E       */
/*                                                                         */
/***************************************************************************/

void Position::SetLon   (long& deg, long& min, long& sec, char *dir) {
 // dir W longitude is negative, E long is positive
 // if - deg => W long => 180 + (180 -(|deg.min.sec|)) => 360 + deg.min.sec
 // if + deg => E long => deg.min.sec
 long lon = abs(deg)*3600L + abs(min)*60L + abs(sec);
 if (lon>HALFLON) lon = HALFLON;
 if (lon<-HALFLON) lon = -HALFLON;
 lon = (dir[0] == 'W' || dir[0] == 'w') ? MAXLON - lon : lon;
 while (lon > MAXLON) lon -= MAXLON;
 while (lon < 0 ) lon += MAXLON;
 Longitude = lon;
}

/***************************************************************************/
/*                                                                         */
/* SetLon - sets Lon from deg, min, sec where W longs are negative         */
/*                                                                         */
/***************************************************************************/

void Position::SetLon   (long& deg, long& min, long& sec) {
 // W longitude is negative, E long is positive
 // if - deg => W long => 180 + (180 -(|deg.min.sec|)) => 360 + deg.min.sec
 // if + deg => E long => deg.min.sec
 long lon = abs(deg)*3600L + abs(min)*60L + abs(sec);
 lon = (deg<0 || min<0 || sec<0) ? MAXLON - lon : lon;
 while (lon > MAXLON) lon -= MAXLON;
 while (lon < 0 ) lon += MAXLON;
 Longitude = lon;
}

/***************************************************************************/
/*                                                                         */
/* SetLatLon - sets both Lat and Lon from pair of longs                    */
/*                                                                         */
/***************************************************************************/

void    Position::SetLatLon (long lat, long lon) {
 SetLat (lat);
 SetLon (lon);
}

/***************************************************************************/
/*                                                                         */
/* FormatPosition - fills string with displayable form of position         */
/*                                                                         */
/* requires a string of length: 24 bytes: N 73 32 13  E 123 43 12          */
/*                                                                         */
/***************************************************************************/

void  Position::FormatPosition (char* line, unsigned int maxlen) {
 long latdeg, latmin, latsec, londeg, lonmin, lonsec;
 char latdir[2], londir[2];

 cvdlat (latdeg, latmin, latsec, latdir, Latitude);
 cvdlon (londeg, lonmin, lonsec, londir, Longitude);

 sprintf_s (line, maxlen, "%c %02ld %02ld %02ld  %c %03ld %02ld %02ld",
          latdir[0], latdeg, latmin, latsec, londir[0], londeg, lonmin, lonsec);
}

/***************************************************************************/
/*                                                                         */
/* PositionToInt - converts a Position into long values for updating        */
/*                                                                         */
/***************************************************************************/

void  Position::PositionToLong (
                 long& latdeg, long& latmin, long& latsec, char *latdir,
                 long& londeg, long& lonmin, long& lonsec, char *londir) {
 cvdlat (latdeg, latmin, latsec, latdir, Latitude);
 cvdlon (londeg, lonmin, lonsec, londir, Longitude);
}

/***************************************************************************/
/*                                                                         */
/* Operator = assigns passed position into this position                   */
/*                                                                         */
/***************************************************************************/

Position&  Position::operator= (Position& pos) {
 Latitude  = pos.Latitude;
 Longitude = pos.Longitude;
 return *this;
}

/***************************************************************************/
/*                                                                         */
/* cvdlat: long latitude in sec converted to deg, min, sec and N/S letter  */
/*                                                                         */
/***************************************************************************/

void cvdlat (long &deg, long &min, long &sec, char *ns, long &llat) {
 long x = labs(llat);

 if (x > TOPLAT) x = TOPLAT;       // force into range
 else if (x < BOTLAT) x = BOTLAT;

 deg = (long) (x/3600);
 x -= deg*3600L;
 min = (long) (x/60);
 sec = (long) (x - min*60L);
 if (llat<0) {
  ns[0]='S';
  ns[1]=0;
 }
 else {
  ns[0]='N';
  ns[1]=0;
 }
}

/***************************************************************************/
/*                                                                         */
/* cvdlon: converts long lon in sec to deg, min, sec and E/W letter        */
/*                                                                         */
/***************************************************************************/

void cvdlon (long &deg, long &min, long &sec, char *ew, long &llon) {
 long x = llon;

 while (x > MAXLON) x -= MAXLON; // guarantee in range
 while (x < 0) x += MAXLON;

 // if - deg => W long => 180 + (180 -(|deg.min.sec|)) => 360 + deg.min.sec
 // if + deg => E long => deg.min.sec

 if (x > HALFLON) {
  ew[0]='W';
  ew[1]=0;
  x = MAXLON - x;
 }
 else {
  ew[0]='E';
  ew[1]=0;
 }
 deg = (long) (x/3600);
 x -= deg*3600L;
 min = (long) (x/60);
 sec = (long) (x - min*60L);
}
